import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.print.*;
import javax.swing.*;
import java.awt.BorderLayout;
import java.util.Scanner;
import java.util.Random;
import java.awt.event.*;
import java.awt.Toolkit;
import java.awt.PrintJob;
/**
 * Die Klasse Kunstwerk kann dazu verwdendet werden Algorihtmen zum erschaffen eines Kunstverkes zu benutzten. Dazu muss man in der Methode erschaffeKunstwerk()
 * Algorihmen kombinieren um auf der zuvor erstellten Zeichenfläche zu malen.
 * 
 * @author  Henning Ainödhofer
 * @version 28.03.2017
 */
public class Kunstwerk
{
    /**
     * Attribut, das als Zeichenfläche dient.
     */
    Lainwand bild;

    /**
     * Konstruktor der Klasse Kunstwerk. 
     * @param width Breite des Kunstwerkes
     * @param hight Höhe des Kunstwkeres
     * @param withGrid Angabe das Kunstwerk mit oder ohne Raster dargestellt werden soll.
     */
    public Kunstwerk(int width, int hight, boolean withGrid)
    {
        bild = new Lainwand(width, hight, withGrid);

        generateGraphics(width, hight);
    }

    /**
     * Konstruktor der Klasse Kunstwerk. 
     * @param width Breite des Kunstwerkes
     * @param hight Höhe des Kunstwkeres
     * @param withGrid Angabe das Kunstwerk mit oder ohne Raster dargestellt werden soll.
     */
    public Kunstwerk()
    {
        bild = new Lainwand(300, 300, false);

        generateGraphics(300, 300);
    }

    /**
     * Die Methode generateGraphics erzeugt einen Frame, der das Kunstwerk anzeigt.
     * @param width Breite des Rahmens
     * @param hight Höhe des Rahmens
     */
    private void generateGraphics(int width, int hight)
    {
        JFrame window = new JFrame();
        window.add(bild.getGrid());
        window.setSize(width+20, hight+40);
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setResizable(false);
        window.setVisible(true);
    }

    private void bildDrucken(JFrame window)
    {
        Toolkit tk = Toolkit.getDefaultToolkit();
        PrintJob pj = tk.getPrintJob(window, "", null );
    }

    /**
     * Die Methode zeichnet das Bild aufgrund der gespeicherten Pixel neu
     */
    private void aktualisiereBild()
    {
        List<Pixel> tempView = bild.getGrid().getFilledCells();
        tempView.toFirst();
        while(tempView.hasAccess())
        {
            double x = (tempView.getContent()).getX();
            double y = (tempView.getContent()).getY();
            bild.setzePixelfarbe((tempView.getContent()).getColor());
            bild.setzePixel((int)x, (int)y);
            bild.setzePixelfarbe(Color.black);
            tempView.next();
        }
    }

    /**
     * Die Methode öffnet ein Dialogfenster zum einfachen Drucken des Bildes. Hier findet keine Skalierung des Bildes statt.
     */
    public void druckeKunstwerk(){
        PrinterJob pj = PrinterJob.getPrinterJob();
        pj.setJobName("Kunstwerk");

        pj.setPrintable (new Printable() {    
                public int print(Graphics pg, PageFormat pf, int pageNum){
                    if (pageNum > 0){
                        return Printable.NO_SUCH_PAGE;
                    }

                    Graphics2D g2 = (Graphics2D) pg;
                    g2.translate(pf.getImageableX(), pf.getImageableY());
                    bild.getLainwand().getGrid().paint(g2);
                    return Printable.PAGE_EXISTS;
                }
            });
        if (pj.printDialog() == false)
            return;

        try {
            pj.print();
        } catch (PrinterException ex) {
            // handle exception
        }
    }

    /**
     * Die Methode kann ein Raster über das Bild legen.
     * @param withGrid true = Raster an, false = Raster aus
     */
    public void zeigeRaster(boolean withGrid)
    {
        bild.getGrid().setWithGrid(withGrid);
        bild.getGrid().repaint();
    }

    /**
     * Diese Methode setzt die Pixelfarbe auf den übergebenen Wert ohne aber einen Pixel im Bild einzufärben
     * 
     * @param r Rot-Wert
     * @param g Grün-Wert
     * @param b Blau-Wert
     */
    public void setzePixelfarbe(int r, int g, int b)
    {
        bild.setzePixelfarbe(Color.getHSBColor(r, g, b));
    }

    /**
     * Diese Methode setzt die Pixelfarbe auf den übergebenen Wert ohne aber einen Pixel im Bild einzufärben.
     * Möglich sind dabei rot, blau, gruen, gelb, schwarz, weiss
     * 
     * @param String Farbe
     */
    public void setzePixelfarbe(Object col)
    {
        bild.setzePixelfarbe(col);
        Color.
    }

    /**
     * Die Methode faerbePixel nimmt die Position des Pixels und die RBG-Werte entgegen und färbt den Pixel an der Stelle in der Farbe.
     * @param x X-Koordinate
     * @param y Y-Koordinate
     * @param r Rot-Wert
     * @param g Grün-Wert
     * @param b Blau-Wert
     */
    public void faerbePixel(int x, int y, int r, int g, int b)
    {
        bild.setzePixelfarbe(Color.getHSBColor(0, 0, 0));
        bild.setzePixel(x, y);
        System.out.println(x+ ": "+y);
    }

    /**
     * Die Methode faerbePixel nimmt die Position des Pixels und die Farbe vom Typ Color entgegen und färbt den Pixel an der Stelle in der Farbe.
     * @param x X-Koordinate
     * @param y Y-Koordinate
     * @param color Farbe
     */
    public void faerbePixel(int x, int y, Color color)
    {
        bild.setzePixelfarbe(color);
        bild.setzePixel(x, y);
    }

    /**
     * Die Methode faerbePixel nimmt die Position des Pixels und den Namen der Farbe entgegen und färbt den Pixel an der Stelle in der Farbe.
     * Möglich sind dabei rot, blau, gruen, gelb, schwarz, weiss
     * @param x X-Koordinate
     * @param y Y-Koordinate
     * @param String Farbe
     */
    public void faerbePixel(int x, int y, String color)
    {
        bild.setzePixelfarbe(color);
        bild.setzePixel(x, y);
    }

    /**
     * Die Methode zeichnet einen Kreis unter der Angabe des Mittelpunktes und des Radius. 
     * 
     * !!! Beachte: Soll im Bild ein Kreis direkt am Anfang gezeichet werden, muss vorher eine Farbe eingesellt werden !!!
     * 
     * @param x X-Koordinate
     * @param y Y-Koordinate
     * @param r Radius
     */
    public void zeichneKreis(int x, int y, int r)
    {
        bild.zeichneKreis(x, y, r);
    }

    /**
     *  Die Methode zeichnet eine Linie unter der Angabe des Start- und des Endpunktes
     *  
     *  !!! Beachte: Soll im Bild eine Linie direkt am Anfang gezeichet werden, muss vorher eine Farbe eingesellt werden !!!
     *  
     *  @param xStart X-Koordinate Startpunkt
     *  @param yStart Y-Koordinate Startpunkt
     *  @param xEnd X-Koordinate Endpunkt
     *  @param yEnd Y-Koordinate Endpunkt
     */
    public void zeichneLinie(int xStart, int yStart, int xEnd, int yEnd)
    {
        bild.zeichneLinie(xStart, yStart, xEnd, yEnd);
    }

    /**
     * In dieser Methode soll ein Algoritmus entwickelt werden, der das Kunstwerk erschafft. Verwenden Sie dabei folgende Strukturen:
     * 1. mind. eine Zählschleife
     * 2. mind. eine Endlosschleife
     * 3. mind. eine Verzweigung
     * 4. mind. eine Mehrfachauswahl in Kombination mit einer Zufallszahl => 
     *                                      Random zufall = new Random();
     *                                      int Eingabe = zufall.nextInt(10) // Zahl zwischen 0 und 10 jeweils inklusive
     * 5. mind. eine Benutzereingabe über die Konsole =>
     *                                      Scanner sc = new Scanner(System.in);
     *                                      System.out.println( //Benutzereingabe );
     *                                      int Eingabe = sc.nextInt();
     */
    public void erschaffeKunstwerk()
    {
        // Hier soll man nun mit Hilfe von Algorithmen ein neues Kunstwerk erschaffen!
        for (int i = 0; i < 360; i = i + 4)
        {
            zeichneLinie(40, 40, 100, i);
        }
        
        int i = 10;
        while(i < 99)
        {
            zeichneKreis(i, i, i);
            i = i + 1;
        }
    }

    public void Uebung1()
    {
    }

    public void Uebung2()
    {
    }

    public void Uebung3()
    {

    }

    public void Uebung4()
    {
    }

    public void Uebung5()
    {


    }

    public void Uebung6()
    {
    }
}
