import java.awt.Color;
import java.awt.Graphics;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * Die Klasse Anzeige stellt eine Rastergrafik her in der einzelne Pixel gezeichnet werden können. Dazu benutzt die Klasse eine
 * interne Klasse Grid, welche das Raster herstellt und sich um die Aktualisierung bei Benutzereingaben kümmert. In der Klasse
 * Anzeige gibt es foglich einige Methoden, die die übergebenen Informationen lediglich an das Raster weiterreichen.
 * 
 * @author Henning Ainödhofer
 * @version 29.03.2017
 */
public class Lainwand {
    Grid grid;

    /**
     * Innere Klasse Grid, welche sich um die Darstellung kümmert.
     */
    public static class Grid extends JPanel  {
        /**
         * Die Liste filledCells umfasst alle Pixel im Raster, die vom Benutzer dargestellt werden sollen. Sie beinhaltet Objekte
         * vom Typ Pixel
         */
        private List <Pixel> filledCells;

        /**
         * Das Attribut color dient dazu, jedem Pixel eine Farbe zuzuweisen. Zu Beginn wird jeder erstellte Pixel in schwarz
         * dargestellt.
         */
        private Color color;

        /**
         * Das Attribut width gibt die width der Pixelmatrix in mm an.
         */
        private int width;

        /**
         * Das Attribut hight gibt die Hoehe der Pixelmatrix in mm an.
         */
        private int hight;

        /**
         * Das Attribut gibt an, ob mit Grid oder ohne gezeichnet werden soll.
         */
        private boolean withGrid;

        /**
         * Konstruktur der Klasse Grid
         */
        public Grid(int width, int hight, boolean withGrid) {
            this.width = width;
            this.hight = hight;
            this.withGrid = withGrid;
            filledCells = new List<Pixel>();
            color = Color.BLACK;
            this.setOpaque(true);
        }

        /**
         * Die Methode paintComponent kümmert sich um das Zeichnen des Rasters und der Pixel. Sie wird immer aufgerufen, sobald 
         * in einer Methode repaint(); aufgerufen wird. 
         * Die Methode arbeitet die Liste der eingefärbten Pixel ab und stellt diese dar. Anschließend wird auch das Raster neu
         * gezeichnet.
         */
        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            filledCells.toFirst();
            while(filledCells.hasAccess()) {
                int cellX = 3 + ((filledCells.getContent()).x * 3);
                int cellY = 3 + ((filledCells.getContent()).y * 3);
                g.setColor((filledCells.getContent()).getColor());
                g.fillRect(cellX, cellY, 3, 3);
                filledCells.next();
            }

            if(withGrid)
            {
                g.setColor(Color.BLACK);
                g.drawRect(3, 3, width, hight);

                for (int i = 3; i <= width; i = i + 3) 
                {
                    g.drawLine(i, 3, i, hight+3);
                }

                for (int i = 3; i <= hight; i = i + 3) 
                {
                    g.drawLine(3, i, width+3, i);
                }
            }
        }

        /**
         * Die Mehtode fillCell fügt der Liste der gefüllten Pixel einen neuen Pixel hinzu und benötigt dazu die x- und die
         * y-Koordninate und eine Farbe.
         * 
         * @param x X-Koordinate des Pixels
         * @param y Y-Koordinate des Pixels
         * @param color Farbe des Pixels
         */
        public void fillCell(int x, int y) {
            if(x <= this.width && y <= this.hight && x >= 0 && y >=0)
            {
                filledCells.append(new Pixel(x, y, color, true));
                repaint();
            }
        }

        /**
         * Die Methode deleteAllCells löscht alle Pixel aus der Liste der gefüllte Pixel und lässt anschließend das Raster neu
         * zeichnen.
         */
        public void deleteAllCells()
        {
            filledCells.toFirst();
            while(filledCells.hasAccess())
            {
                (filledCells.getContent()).setIsSet(false);
                filledCells.remove();
            }
            repaint();
        }

        /**
         * Die Methode setColor weist dem Attribut eine neue Farbe zu.
         * 
         * @param color Farbe des Pixels
         */
        public void setColor(Color color)
        {
            this.color = color;
        }

        /**
         * Die Methode gibt den Attributwert von width zurück
         * @return width
         */
        public int getwidth()
        {
            return this.width/10;
        }

        /**
         * Die Methode gibt den Attributwert von hight zurück
         * @return hight
         */
        public int getHight()
        {
            return this.hight/10;
        }

        /**
         * Die Methode gibt die Liste mit den gefüllten Zellen zurück
         * @return filledCells
         */
        public List getFilledCells()
        {
            return this.filledCells; 
        }

        /**
         * Die Methode setzt den Attributwert von withGrid auf den übergebenen Wert
         * @param withGrid
         */
        public void setWithGrid(boolean withGrid)
        {
            this.withGrid = withGrid;
        }

        /**
         * Diese Methode gibt den Pixel an der Stelle x, y zurück. Wenn dieser 
         * nicht exisistiert wird null zurück gegeben.
         */
        public Pixel getPixel(int x, int y)
        {
            filledCells.toFirst();
            while(filledCells.hasAccess())
            {
                if(filledCells.getContent().getX() == x && filledCells.getContent().getY() == y)
                {
                    return filledCells.getContent();
                }
                filledCells.next();
            }
            return null;
        }
    }

    /**
     * Konstruktor der Klasse Anzeige. Hier wird der Frame erzeugt und das Raster zugewiesen.
     */
    public Lainwand (int width, int hight, boolean withGrid)
    {
        grid = new Grid(width, hight, withGrid);
    }

    /**
     * Diese Methode gibt das Objekt Zurück, welches die Lainwand beinhaltet
     * @return lainwand
     */
    public Lainwand getLainwand()
    {
        return this;
    }

    /**
     * Diese Methode gibt die Zeichenfläche zurück
     * 
     * @retrun grid
     */
    public Grid getGrid()
    {
        return this.grid;
    }

    /**
     * Die Methode setzePixel faerbt einen Pixel im Raster in der aktuellen Farbe
     * 
     * @param x X-Koordinate des Pixels
     * @param y Y-Koordinate des Pixels
     */
    public void setzePixel(int x, int y)
    {
        grid.fillCell(x, y);
    }

    /**
     * Die Methode deleteAllPixel löscht alle Pixel aus dem Raster
     */
    public void loescheAllePixel()
    {
        grid.deleteAllCells();
    }

    /**
     * Die Methode setzePixelfarbe setzt eine neue Farbe für den nächsten erzeugten Pixel. Mögliche Farben sind:
     * rot, blau, gruen, gelb, schwarz, weiss
     * 
     * @param color Farbe des Pixels
     */
    public void setzePixelfarbe(Object color)
    {
        if (color instanceof String)
        {
            String c = (String) color;
            if(c.equalsIgnoreCase("rot"))grid.setColor(Color.red);
            if(c.equalsIgnoreCase("blau")) grid.setColor(Color.blue);
            if(c.equalsIgnoreCase("gruen")) grid.setColor(Color.green);
            if(c.equalsIgnoreCase("gelb")) grid.setColor(Color.orange);
            if(c.equalsIgnoreCase("schwarz")) grid.setColor(Color.black);
            if(c.equalsIgnoreCase("weiss")) grid.setColor(Color.white);
        }
        else
        {
            Color c = (Color) color;
            grid.setColor(c);
        }
    }

    public Pixel gibPixel(int x, int y)
    {
        return this.getGrid().getPixel(x,y);
    }

    /**
     * Diese Methode zeichnet mit Hilfe des Bresenham-Algorihtmus einen Kreis aus einzelnen Pixeln.
     * 
     * @param x X-Koordinate des Mittelpunktes
     * @param y Y-Koordinate des Mittelpunktes
     * @param radius Radius des Kreises
     */
    public void zeichneKreis(int x, int y,int radius)
    {
        double a, b, d, dx, dxy;
        a = 0;
        b = radius;
        d = 1 - radius;
        dx = 3;
        dxy = -2 * radius + 5;
        while (b >= a) {
            setzePixel((int) (0.5 + x + a), (int) (0.5 + y + b)); // alle 8 Oktanden 
            setzePixel((int) (0.5 + x + b), (int) (0.5 + y + a)); // gleichzeitig zeichnen
            setzePixel((int) (0.5 + x + b), (int) (0.5 + y - a));
            setzePixel((int) (0.5 + x + a), (int) (0.5 + y - b));
            setzePixel((int) (0.5 + x - a), (int) (0.5 + y - b));
            setzePixel((int) (0.5 + x - b), (int) (0.5 + y - a));
            setzePixel((int) (0.5 + x - b), (int) (0.5 + y + a));
            setzePixel((int) (0.5 + x - a), (int) (0.5 + y + b));

            if (d < 0) {
                d = d + dx;
                dx = dx + 2;
                dxy = dxy + 2;
                a++;
            } else {
                d = d + dxy;
                dx = dx + 2;
                dxy = dxy + 4;
                a++;
                b--;
            }
        }
    }

    /**
     *  Die Methode zeichnet eine Linie unter der Angabe des Start- und des Endpunktes
     *  @param xStart X-Koordinate Startpunkt
     *  @param yStart Y-Koordinate Startpunkt
     *  @param xEnd X-Koordinate Endpunkt
     *  @param yEnd Y-Koordinate Endpunkt
     */
    public void zeichneLinie(int xStart, int yStart, int xEnd, int yEnd)
    {
        int xIncrement = 1;
        int yIncrement = 1;
        int dy = 2*(yEnd-yStart);
        int dx = 2*(xStart-xEnd);
        int tmp;

        if ( xStart > xEnd ) {      // Spiegeln an Y-Achse
            xIncrement = -1;
            dx = -dx;
        }

        if ( yStart > yEnd ) {      // Spiegeln an X-Achse
            yIncrement= -1;
            dy= -dy;
        }

        int e = 2*dy + dx;
        int x = xStart;           // Startpunkte setzen
        int y = yStart;

        if ( dy < -dx )       // Steigung < 1
        {      
            while( x != (xEnd+1) )
            {
                e += dy;
                if ( e > 0) 
                {
                    e += dx;
                    y += yIncrement;
                }
                setzePixel(x,y);
                x += xIncrement;
            }
        }
        else // ( dy >= -dx )   Steigung >=1
        {
            // an der Winkelhalbierenden spiegeln
            tmp = -dx;
            dx = -dy;
            dy = tmp;

            e = 2*dy + dx;

            while( y != (yEnd+1) )
            {
                e += dy;
                if( e > 0 ) {
                    e += dx;
                    x += xIncrement;
                }
                setzePixel(x,y);
                y += yIncrement;
            }
        }
    }
}
